/*  @(#)35  1.2 src/provisioning/scenario/src/com/tivoli/am/fim/provisioning/scenario/TestTAM.java, amws.provisioning, fim611, 060808b 2/24/05 19:05:15  */

/*
 *
 * File Name: TestTAM.java
 *
 * IBM Confidential
 * Object Code Only Source Materials
 * 5747-SM3
 * (c) Copyright International Business Machines Corp. 2004
 * The source code for this program is not published or otherwise divested
 * of its trade secrets, irrespective of what has been deposited with the
 * U.S. Copyright Office.
 */

package com.tivoli.am.fim.provisioning.scenario;

import java.net.URL;
import java.util.Hashtable;
import java.net.MalformedURLException;


import com.tivoli.pd.jutil.PDContext;
import com.tivoli.pd.jutil.PDMessages;
import com.tivoli.pd.jutil.PDRgyUserName;
import com.tivoli.pd.jadmin.PDUser;
import com.tivoli.pd.jutil.PDException;

import javax.naming.Context;
import javax.naming.directory.InitialDirContext;
import javax.naming.directory.DirContext;
import javax.naming.directory.AttributeModificationException;
import javax.naming.NamingException;
import javax.naming.NamingEnumeration;
import javax.naming.directory.ModificationItem;
import javax.naming.directory.Attribute;
import javax.naming.directory.Attributes;
import javax.naming.directory.BasicAttribute;
import javax.naming.directory.BasicAttributes;
import javax.naming.directory.SearchResult;

/**
 * Class that does the actual TAM/LDAP partof the provision, deprovision, modifyProvisionedState and modifyProvisionedParameters 
 * WS-Provisioning operations.
 */
public class TestTAM
{
    public static final String TestTAM_java_sourceCodeID = "$Id: @(#)35    1.2  src/provisioning/scenario/src/com/tivoli/am/fim/provisioning/scenario/TestTAM.java, amws.provisioning, fim611, 060808b  2/24/05  19:05:15 $";

    public static final String TestTAM_java_copyright = com.tivoli.am.fim.constants.FIMCopyrightConstants.FULL_COPYRIGHT;

    private String _configURL = null;
    private String _tamAdmin = null;
    private String _tamPassword = null;
    private String _ldapUniqueIdentifier = null;
    private String _ldapServer = null;
    private String _ldapAdminUsername = null;
    private String _ldapAdminPwd = null;
	private boolean _debug = false;

	/**
	 * Constructor that sets up required variables
	 *
	 * @param configURL The PDJrte SvrSslCfg config URL required to communicate with TAM
	 * @param adminUsername The TAM administrator name
	 * @param adminPassword The TAM administrator password
	 * @param ldapUniqueIdentifier An ldap attribute that can be used to distinguish one entry from another
	 * @param ldapServer The LDAP server to update
	 * @param ldapAdminUsername The LDAP administrator name
	 * @param ldapAdminPwd The LDAP administrator password
	 */
    public TestTAM(String configURL, String adminUsername, String adminPassword, String ldapUniqueIdentifier,
                   String ldapServer, String ldapAdminUsername, String ldapAdminPwd)
    {

        _configURL = configURL;
        _tamAdmin = adminUsername;
        _tamPassword = adminPassword;
        _ldapUniqueIdentifier = ldapUniqueIdentifier;
        _ldapServer = ldapServer;
        _ldapAdminUsername = ldapAdminUsername;
        _ldapAdminPwd = ldapAdminPwd;

    }

	/**
	 * Constructor that sets up required variables and turns on debugging
	 *
	 * @param configURL The PDJrte SvrSslCfg config URL required to communicate with TAM
	 * @param adminUsername The TAM administrator name
	 * @param adminPassword The TAM administrator password
	 * @param ldapUniqueIdentifier An ldap attribute that can be used to distinguish one entry from another
	 * @param ldapServer The LDAP server to update
	 * @param ldapAdminUsername The LDAP administrator name
	 * @param ldapAdminPwd The LDAP administrator password
	 * @param debug Boolean value to indicate whether or not to output debug messages
	 */
	public TestTAM(String configURL, String adminUsername, String adminPassword, String ldapUniqueIdentifier,
                   String ldapServer, String ldapAdminUsername, String ldapAdminPwd, boolean debug)
    {

        _configURL = configURL;
        _tamAdmin = adminUsername;
        _tamPassword = adminPassword;
        _ldapUniqueIdentifier = ldapUniqueIdentifier;
        _ldapServer = ldapServer;
        _ldapAdminUsername = ldapAdminUsername;
        _ldapAdminPwd = ldapAdminPwd;

		_debug = debug;

    }

    /** Public method that creates a new user in TAM and updates LDAP with a uniques identifier
     * 
     * @param username The user to create
     * @param dn The dn of the user to create
     * @param password The password of the user to create
     * @param identifier The unique identifier of the user, used in ldap
     * 
     * @throws Exception If a required parameter is null or a PDException/ LdapException occurs
     * 
     */
    public boolean provision(String username, String dn, String password, String identifier) throws Exception
    {
		if (_debug)
		{
			System.out.println("TestTAM.provision() Entry username : " + username + " , identifier : " + identifier + " , DN : " + dn);
		}

        // First ensure that all required values exist

        StringBuffer nullVals = new StringBuffer();

        if (_configURL == null)
        {
            nullVals.append("TAM Config URL");
        }

        if (_tamAdmin == null)
        {
            if (nullVals.length() <= 0)
            {
                nullVals.append("TAM Admin User");
            }
            else
            {
                nullVals.append(", TAM Admin User");
            }
        }

        if (_tamPassword == null)
        {
            if (nullVals.length() <= 0)
            {
                nullVals.append("TAM Admin Password");
            }
            else
            {
                nullVals.append(", TAM Admin Password");
            }
        }

        if (_ldapUniqueIdentifier == null)
        {
            if (nullVals.length() <= 0)
            {
                nullVals.append("LDAP Unique Identifier Attribute");
            }
            else
            {
                nullVals.append(", LDAP Unique Identifier Attribute");
            }
        }

        if (_ldapServer == null)
        {
            if (nullVals.length() <= 0)
            {
                nullVals.append("LDAP Server");
            }
            else
            {
                nullVals.append(", LDAP Server");
            }
        }

        if (_ldapAdminUsername == null)
        {
            if (nullVals.length() <= 0)
            {
                nullVals.append("LDAP Admin User");
            }
            else
            {
                nullVals.append(", LDAP Admin User");
            }
        }

        if (_ldapAdminPwd == null)
        {
            if (nullVals.length() <= 0)
            {
                nullVals.append("LDAP Admin Password");
            }
            else
            {
                nullVals.append(", LDAP Admin Password");
            }
        }

        if (username == null)
        {
            if (nullVals.length() <= 0)
            {
                nullVals.append("TAM User to create");
            }
            else
            {
                nullVals.append(", TAM User to create");
            }
        }

        if (dn == null)
        {
            if (nullVals.length() <= 0)
            {
                nullVals.append("DN of User to create");
            }
            else
            {
                nullVals.append(", DN of User to create");
            }
        }

        if (password == null)
        {
            if (nullVals.length() <= 0)
            {
                nullVals.append("Password of User to create");
            }
            else
            {
                nullVals.append(", Password of User to create");
            }
        }

        if (identifier == null)
        {
            if (nullVals.length() <= 0)
            {
                nullVals.append("LDAP Unique Identifier of User to create");
            }
            else
            {
                nullVals.append(", LDAP Unique Identifier of User to create");
            }
        }

        // If any were null then log a message and return false
        if (nullVals.length() > 0)
        {
			System.out.println("TestTAM.provision() The following values are required to be set prior to provisioning a user in TAM : " + nullVals.toString());

			if (_debug)
			{
				System.out.println("TestTAM.provision() Exit Return : " + false);
			}

			return false;
        }
		
		// Create a URL from the _configURL string
        URL ctxURL = null;

        try
        {
            ctxURL = new URL(_configURL);
        }
        catch(MalformedURLException mue)
        {
			System.out.println("TestTAM.provision() The TAM Config URL string is a Malformed URL : " + _configURL); 

			if (_debug)
			{
				System.out.println("TestTAM.provision() Exit Return : " + false);
			}

			return false;
        }

        PDContext ctx = null;

        try
        {
            // create the context
            ctx = new PDContext(_tamAdmin, _tamPassword.toCharArray(), ctxURL);
    
            PDMessages msgs = new PDMessages();
    
            msgs.clear();

            // Create the new user
            PDUser.createUser(
                ctx, 
                username, 
                new PDRgyUserName(dn, username, username), 
                null, 
                password.toCharArray(),
                null,
                false,
                true,
                msgs);

            // Make sure we close the context
            ctx.close();
		}
		catch(PDException pde)
		{
			System.out.println("TestTAM.provision() A PDException was encountered while creating the TAM user : " + username + " : " + pde.toString());

			if (_debug)
			{
				System.out.println("TestTAM.provision() Exit Return : " + false);
			}

			return false;
		}
		finally
        {
            try
            {
                ctx.close();
            }
            catch(Exception e)
            {
                // Not much we can do
            }
        }

        // Now add the identifier string to the ldap ePerson uniqueIdentifier attribute
		Hashtable ldapenv = new Hashtable( );

        DirContext ldapctx = null;

		ldapenv.put( Context.INITIAL_CONTEXT_FACTORY, "com.sun.jndi.ldap.LdapCtxFactory" ); 
        ldapenv.put( Context.URL_PKG_PREFIXES, "com.ibm.jndi" );
		ldapenv.put( Context.PROVIDER_URL, _ldapServer );
        ldapenv.put( Context.SECURITY_PRINCIPAL, _ldapAdminUsername );
        ldapenv.put( Context.SECURITY_CREDENTIALS, _ldapAdminPwd );

        ldapctx = new InitialDirContext ( ldapenv );
            
        ModificationItem[] mods = new ModificationItem[1];
        Attribute mod0 = new BasicAttribute( _ldapUniqueIdentifier, identifier );

        /* Update mail attribute */
        mods[0] = new ModificationItem( DirContext.REPLACE_ATTRIBUTE, mod0 );
        
		// Do the actual modification
		try
		{
			ldapctx.modifyAttributes( dn, mods );
		}
		catch(AttributeModificationException ame)
		{
			System.out.println("TestTAM.provision() Encountered an AttributeModificationException while attempting to update the ldap uniqueIdentifier : " + ame.toString());

			if (_debug)
			{
				System.out.println("TestTAM.provision() Exit Return : " + false);
			}

			return false;
		}
		catch(NamingException ne)
		{
			System.out.println("TestTAM.provision() Encountered a NamingException while attempting to update the ldap uniqueIdentifier : " + ne.toString());

			if (_debug)
			{
				System.out.println("TestTAM.provision() Exit Return : " + false);
			}

			return false;
		}
		finally
        {
            try
            {
                ldapctx.close();
            }
            catch(Exception e)
            {
                // Not much we can do
            }
        }

		// Ensure that we close the ldap context
        ldapctx.close();

        if (_debug)
		{
			System.out.println("TestTAM.provision() Exit Return : " + true);
		}

		// Everything OK
		return true;
    }

	/**
	 * Public method to deprovision a TAM user.
	 * It looks for the username in ldap using an ldap unique identifier.
	 *
	 * @param identifier The ldap unique identifier
	 * @param suffix The ldap suffix to search the user for.
	 *
	 * @return boolean Whether or not the deprovisioned succeeded
	 */
    public boolean deprovision(String identifier, String suffix)
    {
		if (_debug)
		{
			System.out.println("TestTAM.deprovision() identifier : " + identifier + " , suffix : " + suffix);
		}

        // First ensure that all required values exist
        StringBuffer nullVals = new StringBuffer();

        if (_configURL == null)
        {
            nullVals.append("TAM Config URL");
        }

        if (_tamAdmin == null)
        {
            if (nullVals.length() <= 0)
            {
                nullVals.append("TAM Admin User");
            }
            else
            {
                nullVals.append(", TAM Admin User");
            }
        }

        if (_tamPassword == null)
        {
            if (nullVals.length() <= 0)
            {
                nullVals.append("TAM Admin Password");
            }
            else
            {
                nullVals.append(", TAM Admin Password");
            }
        }

        if (_ldapUniqueIdentifier == null)
        {
            if (nullVals.length() <= 0)
            {
                nullVals.append("LDAP Unique Identifier Attribute");
            }
            else
            {
                nullVals.append(", LDAP Unique Identifier Attribute");
            }
        }

        if (_ldapServer == null)
        {
            if (nullVals.length() <= 0)
            {
                nullVals.append("LDAP Server");
            }
            else
            {
                nullVals.append(", LDAP Server");
            }
        }

        if (_ldapAdminUsername == null)
        {
            if (nullVals.length() <= 0)
            {
                nullVals.append("LDAP Admin User");
            }
            else
            {
                nullVals.append(", LDAP Admin User");
            }
        }

        if (_ldapAdminPwd == null)
        {
            if (nullVals.length() <= 0)
            {
                nullVals.append("LDAP Admin Password");
            }
            else
            {
                nullVals.append(", LDAP Admin Password");
            }
        }

        if (suffix == null)
        {
            if (nullVals.length() <= 0)
            {
                nullVals.append("Suffix of User to create");
            }
            else
            {
                nullVals.append(", Suffix of User to create");
            }
        }

        if (identifier == null)
        {
            if (nullVals.length() <= 0)
            {
                nullVals.append("LDAP Unique Identifier of User to create");
            }
            else
            {
                nullVals.append(", LDAP Unique Identifier of User to create");
            }
        }

		// If any were null then log a message and return false
        if (nullVals.length() > 0)
        {
			System.out.println("TestTAM.deprovision() The following values are required to be set prior to deprovisioning a user in TAM : " + nullVals.toString());

			if (_debug)
			{
				System.out.println("TestTAM.deprovision() Exit Return : " + false);
			}

			return false;
        }

        // Create a URL from the _configURL string
        URL ctxURL = null;

        try
        {
            ctxURL = new URL(_configURL);
        }
        catch(MalformedURLException mue)
        {
			System.out.println("TestTAM.deprovision() The TAM Config URL string is a Malformed URL : " + _configURL); 

			if (_debug)
			{
				System.out.println("TestTAM.deprovision() Exit Return : " + false);
			}

			return false;
        }

        PDContext ctx = null;

		String uidVal = null;

		// Get the username from ldap using unique identifier
        try
        {
            uidVal = findUidFromIdentifier(identifier, suffix);
		}
		catch(NamingException ne)
		{
			System.out.println("TestTAM.deprovision() Encountered a NamingException while attempting to get the DN from the ldap uniqueIdentifier : " + ne.toString());

			if (_debug)
			{
				System.out.println("TestTAM.deprovision() Exit Return : " + false);
			}

			return false;
		}	

		// If could find the username then delete it
        if (uidVal != null)
        {
			if (_debug)
			{
				System.out.println("TestTAM.deprovision() About to delete the TAM user : " + uidVal);
			}

			try
			{
                // create the context
                ctx = new PDContext(_tamAdmin, _tamPassword.toCharArray(), ctxURL);
    
                PDMessages msgs = new PDMessages();
    
                msgs.clear();
    
                // Create the new user
                PDUser.deleteUser(ctx, uidVal, true, msgs);
                    
                // Make sure we close the context
                ctx.close();
            }
      
			catch(PDException pde)
			{
				System.out.println("TestTAM.deprovision() A PDException was encountered while deleting the TAM user : " + uidVal + " : " + pde.toString());

				if (_debug)
				{
					System.out.println("TestTAM.deprovision() Exit Return : " + false);
				}

				return false;
			}
			finally
			{
				try
				{
					ctx.close();
				}
				catch(Exception e)
				{
					// Not much we can do
				}
			}
		}
		else // Null uidVal
		{
			System.out.println("TestTAM.deprovision() No TAM user could be found in ldap for the unique identifier.");
			
			if (_debug)
			{
				System.out.println("TestTAM.deprovision() Exit Return : " + false);
			}

			return false;
		}

		// Everything OK
		if (_debug)
		{
			System.out.println("TestTAM.deprovision() Exit Return : " + true);
		}
        return true;
    }

	/**
	 * Public method that modifies the account-valid state for a TAM user.
	 * It finds the TAM username from an ldap unique identifier and then calls TAM to make update.
	 *
	 * @param identifier The ldap unique identifier for the user to modify
	 * @param suffix The ldap suffix to search for the username in
	 * @param isValidStr A string representation of true or false for what to change the account-valid to
	 *
	 * @return boolean Whether or not the modification succeeded
	 */
    public boolean modifyProvisionedState(String identifier, String suffix, String isValidStr)
    {
		if (_debug)
		{
			System.out.println("TestTAM.modifyProvisionedState() Entry identifier : " + identifier + " , suffix : " + suffix + " , isValidStr : " + isValidStr);
		}

        // First ensure that all required values exist
        StringBuffer nullVals = new StringBuffer();

        if (_configURL == null)
        {
            nullVals.append("TAM Config URL");
        }

        if (_tamAdmin == null)
        {
            if (nullVals.length() <= 0)
            {
                nullVals.append("TAM Admin User");
            }
            else
            {
                nullVals.append(", TAM Admin User");
            }
        }

        if (_tamPassword == null)
        {
            if (nullVals.length() <= 0)
            {
                nullVals.append("TAM Admin Password");
            }
            else
            {
                nullVals.append(", TAM Admin Password");
            }
        }

        if (_ldapUniqueIdentifier == null)
        {
            if (nullVals.length() <= 0)
            {
                nullVals.append("LDAP Unique Identifier Attribute");
            }
            else
            {
                nullVals.append(", LDAP Unique Identifier Attribute");
            }
        }

        if (_ldapServer == null)
        {
            if (nullVals.length() <= 0)
            {
                nullVals.append("LDAP Server");
            }
            else
            {
                nullVals.append(", LDAP Server");
            }
        }

        if (_ldapAdminUsername == null)
        {
            if (nullVals.length() <= 0)
            {
                nullVals.append("LDAP Admin User");
            }
            else
            {
                nullVals.append(", LDAP Admin User");
            }
        }

        if (_ldapAdminPwd == null)
        {
            if (nullVals.length() <= 0)
            {
                nullVals.append("LDAP Admin Password");
            }
            else
            {
                nullVals.append(", LDAP Admin Password");
            }
        }

        if (suffix == null)
        {
            if (nullVals.length() <= 0)
            {
                nullVals.append("Suffix of User to create");
            }
            else
            {
                nullVals.append(", Suffix of User to create");
            }
        }

        if (identifier == null)
        {
            if (nullVals.length() <= 0)
            {
                nullVals.append("LDAP Unique Identifier of User to create");
            }
            else
            {
                nullVals.append(", LDAP Unique Identifier of User to create");
            }
        }

        if (isValidStr == null)
        {
            if (nullVals.length() <= 0)
            {
                nullVals.append("Account-Valid");
            }
            else
            {
                nullVals.append(", Account-Valid");
            }
        }

		// If any were null then log a message and return false
        if (nullVals.length() > 0)
        {
			System.out.println("TestTAM.modifyProvisionedState() The following values are required to be set prior to modifying the state of a user in TAM : " + nullVals.toString());

			if (_debug)
			{
				System.out.println("TestTAM.modifyProvisionedState() Exit Return : " + false);
			}

			return false;
        }

        // Create a URL from the _configURL string
        URL ctxURL = null;

        try
        {
            ctxURL = new URL(_configURL);
        }
        catch(MalformedURLException mue)
        {
			System.out.println("TestTAM.modifyProvisionedState() The TAM Config URL string is a Malformed URL : " + _configURL); 

			if (_debug)
			{
				System.out.println("TestTAM.modifyProvisionedState() Exit Return : " + false);
			}

			return false;
        }

        PDContext ctx = null;
        
		// Find out whether the account is to be valid or invalid
		boolean isValid = true;

        if (isValidStr.equalsIgnoreCase("false"))
        {
            isValid = false;
        }

		String uidVal = null;

		// Get the username to modify from ldap using the unique identifier
        try
        {
            uidVal = findUidFromIdentifier(identifier, suffix);
		}
		catch(NamingException ne)
		{
			System.out.println("TestTAM.modifyProvisionedState() Encountered a NamingException while attempting to get the DN from the ldap uniqueIdentifier : " + ne.toString());

			if (_debug)
			{
				System.out.println("TestTAM.modifyProvisionedState() Exit Return : " + false);
			}

			return false;
		}	

		// Make the modification if the user was found
		if (uidVal != null)
		{
			if (_debug)
			{
				System.out.println("TestTAM.modifyProvisionedState() About to set the account-valid to " + isValid + " for the TAM user : " + uidVal);
			}
			
			try
			{
				// create the context
				ctx = new PDContext(_tamAdmin, _tamPassword.toCharArray(), ctxURL);
    
				PDMessages msgs = new PDMessages();
    
				msgs.clear();
    
				// Create the new user
			    PDUser.setAccountValid(ctx, uidVal, isValid, msgs);
                    
			    // Make sure we close the context
			    ctx.close();
			}
			catch(PDException pde)
			{
				System.out.println("TestTAM.modifyProvisionedState() A PDException was encountered while modifying the state of the TAM user : " + uidVal + " : " + pde.toString());

				if (_debug)
				{
					System.out.println("TestTAM.modifyProvisionedState() Exit Return : " + false);
				}
			}
			finally
			{
				try
				{
					ctx.close();
				}
				catch(Exception e)
				{
					// Not much we can do
				}
			}
		}
		else // Null uidVal
		{
			System.out.println("TestTAM.modifyProvisionedState() No TAM user could be found in ldap for the unique identifier.");
			
			if (_debug)
			{
				System.out.println("TestTAM.modifyProvisionedState() Exit Return : " + false);
			}

			return false;
		}

		if (_debug)
		{
			System.out.println("TestTAM.modifyProvisionedState() Exit Return : " + true);
		}

        return true;
    }

	/**
	 * Public method used to update an ldap parameter for a user.
	 * The method first looks for the DN to update using the ldap unique identifier then updates it.
	 *
	 * @param identifier The ldap unique identifier
	 * @param suffix The ldap suffix to search in
	 * @param attToUpdate The ldap attribute to update
	 * @param newValue The new value for the attribute
	 *
	 * @return boolean Whether or not the modification succeeded
	 */
    public boolean modifyProvisionedParameters(String identifier, String suffix, String attToUpdate, String newValue)
    {
		if (_debug)
		{
			System.out.println("TestTAM.modifyProvisionedParameters() Entry identifier : " + identifier + " , suffix : " + suffix + " , attToUpdate : " + attToUpdate + " , newValue : " + newValue);
		}

        // First ensure that all required values exist
        StringBuffer nullVals = new StringBuffer();

        if (_ldapUniqueIdentifier == null)
        {
            if (nullVals.length() <= 0)
            {
                nullVals.append("LDAP Unique Identifier Attribute");
            }
            else
            {
                nullVals.append(", LDAP Unique Identifier Attribute");
            }
        }

        if (_ldapServer == null)
        {
            if (nullVals.length() <= 0)
            {
                nullVals.append("LDAP Server");
            }
            else
            {
                nullVals.append(", LDAP Server");
            }
        }

        if (_ldapAdminUsername == null)
        {
            if (nullVals.length() <= 0)
            {
                nullVals.append("LDAP Admin User");
            }
            else
            {
                nullVals.append(", LDAP Admin User");
            }
        }

        if (_ldapAdminPwd == null)
        {
            if (nullVals.length() <= 0)
            {
                nullVals.append("LDAP Admin Password");
            }
            else
            {
                nullVals.append(", LDAP Admin Password");
            }
        }

        if (suffix == null)
        {
            if (nullVals.length() <= 0)
            {
                nullVals.append("Suffix of User to create");
            }
            else
            {
                nullVals.append(", Suffix of User to create");
            }
        }

        if (identifier == null)
        {
            if (nullVals.length() <= 0)
            {
                nullVals.append("LDAP Unique Identifier of User to create");
            }
            else
            {
                nullVals.append(", LDAP Unique Identifier of User to create");
            }
        }

        if (attToUpdate == null)
        {
            if (nullVals.length() <= 0)
            {
                nullVals.append("The attribute to update");
            }
            else
            {
                nullVals.append(", The attribute to update");
            }
        }

		if (newValue == null)
        {
            if (nullVals.length() <= 0)
            {
                nullVals.append("New attribute value");
            }
            else
            {
                nullVals.append(", New attribute value");
            }
        }

		// If any were null then log a message and return false
        if (nullVals.length() > 0)
        {
			System.out.println("TestTAM.modifyProvisionedParameters() The following values are required to be set prior to modifying a parameter of a user : " + nullVals.toString());

			if (_debug)
			{
				System.out.println("TestTAM.modifyProvisionedParameters() Exit Return : " + false);
			}

			return false;
        }

		// Create the ldap DirContext
		DirContext ldapctx = null;

		try
		{
			ldapctx = getLdapContext();
		}
		catch(NamingException ne)
		{
			System.out.println("TestTAM.modifyProvisionedParameters() Encountered a NamingException while attempting to create the ldap context : " + ne.toString());

			if (_debug)
			{
				System.out.println("TestTAM.modifyProvisionedParameters() Exit Return : " + false);
			}

			return false;
		}
		
		// Setup the ldap search criteria
        Attribute search0 = new BasicAttribute( _ldapUniqueIdentifier, identifier );
        Attribute search1 = new BasicAttribute( "objectclass", "ePerson" );

        Attributes searchAtts = new BasicAttributes();
        searchAtts.put(search0);
        searchAtts.put(search1);
		
		NamingEnumeration results = null;
		String dn = null;

		// Do the ldap search
		try
		{
			results = ldapctx.search(suffix, searchAtts, null);
		
			// Find the DN from the search results
			while (results.hasMore())
			{
				SearchResult result = (SearchResult)results.next();

				if (result.isRelative())
				{
					// Get the name and create the DN using the suffix
					String name = result.getName();

					dn = name + "," + suffix;
				}

				// If already found then break loop
				if (dn != null)
				{
					break;
				}
			}
		}
		catch(NamingException ne)
		{
			System.out.println("TestTAM.modifyProvisionedParameters() Encountered a NamingException while attempting to find the DN for the ldap unique identifier : " + ne.toString());

			if (_debug)
			{
				System.out.println("TestTAM.modifyProvisionedParameters() Exit Return : " + false);
			}

			return false;
		}
		if (dn != null)
		{
			if (_debug)
			{
				System.out.println("TestTAM.modifyProvisionedParameters() About to modify DN : " + dn);
			}

			ModificationItem[] mods = new ModificationItem[1];
			Attribute mod0 = new BasicAttribute( attToUpdate, newValue );
    
			mods[0] = new ModificationItem( DirContext.REPLACE_ATTRIBUTE, mod0 );
            
			try
			{
				ldapctx.modifyAttributes( dn, mods );
			}
			catch(NamingException ne)
			{
				System.out.println("TestTAM.modifyProvisionedParameters() Encountered a NamingException while attempting to modify the attribute for the DN : " + ne.toString());

				if (_debug)
				{
					System.out.println("TestTAM.modifyProvisionedParameters() Exit Return : " + false);
				}

				return false;
			}
		}
        else
        {
            System.out.println("TestTAM.modifyProvisionedParameters() User with unique identifier " + identifier + " not found in registry.");
			
			if (_debug)
			{
				System.out.println("TestTAM.modifyProvisionedParameters() Exit Return : " + false);
			}

			return false;
        }
        
        try
		{
			ldapctx.close();
		}
		catch(NamingException ne)
		{
			System.out.println("TestTAM.modifyProvisionedParameters() Encountered a NamingException while attempting to close the ldap DirContext : " + ne.toString());
		}

		if (_debug)
		{
			System.out.println("TestTAM.modifyProvisionedParameters() Exit Return : " + true);
		}
        return true;
    }

	/**
	* Private method that will create an ldap DirContext.
	* 
	* @throws NamingException
	*
	* @return DirContext The InitialDirContext
	*/
    private DirContext getLdapContext() throws NamingException
    {
		if (_debug)
		{
			System.out.println("TAMTest.getLdapContext() Entry");
		}

        // First find the username from the ldap unique identifier
        Hashtable ldapenv = new Hashtable( );

        DirContext ldapctx = null;

        ldapenv.put( Context.INITIAL_CONTEXT_FACTORY, "com.sun.jndi.ldap.LdapCtxFactory" ); 
        ldapenv.put( Context.URL_PKG_PREFIXES, "com.ibm.jndi" );
        ldapenv.put( Context.PROVIDER_URL, _ldapServer );
        ldapenv.put( Context.SECURITY_PRINCIPAL, _ldapAdminUsername );
        ldapenv.put( Context.SECURITY_CREDENTIALS, _ldapAdminPwd );

        ldapctx = new InitialDirContext ( ldapenv );

		if (_debug)
		{
			System.out.println("TAMTest.getLdapContext() Exit");
		}
        return ldapctx;
    }

	/**
	* Private method that will search ldap for a unique identifier and return the UID that corresponds.
	*
	* @param identifier The unique identifier to search for
	* @param suffix The ldap suffix to search within
	* 
	* @throws NamingException
	*
	* @return String The UID that is found. Can be null if unique identifier does not exist
	*/
    private String findUidFromIdentifier(String identifier, String suffix) throws NamingException
    {
		if (_debug)
		{
			System.out.println("TAMTest.findUidFromIdentifier() Entry identifier : " + identifier + " , suffix : " + suffix);
		}

		// First create the ldap context
        DirContext ldapctx = getLdapContext();
		
		// The search attributes
        Attribute search0 = new BasicAttribute( _ldapUniqueIdentifier, identifier );
        Attribute search1 = new BasicAttribute( "objectclass", "ePerson" );

        Attributes searchAtts = new BasicAttributes();
        searchAtts.put(search0);
        searchAtts.put(search1);

		// The attributes we want the search to return
        String[] attsToReturn = { "uid" };

		// Do the search
        NamingEnumeration results = ldapctx.search(suffix, searchAtts, attsToReturn);

        ldapctx.close();

        String uidVal = null;

		// Look through the results for the UID
        while (results.hasMore())
        {
            SearchResult result = (SearchResult)results.next();
            Attributes atts = result.getAttributes();

            if (atts != null)
            {
                Attribute uidAtt = atts.get("uid");

                if (uidAtt != null)
                {
                    uidVal = (String)uidAtt.get(0);
                }
            }

			// Found it so end loop
            if (uidVal != null)
            {
                break;
            }
        }
		
		// Return the UID
		if (_debug)
		{
			System.out.println("TAMTest.findUidFromIdentifier() Exit Return : " + uidVal);
		}
        return uidVal;
    }

}
